import unittest
import utils
import struct
import driver

class ProtocolTest(unittest.TestCase):

    def connect(self):
        drv = driver.Driver("10.10.20.124", 11000)
        return drv


    def testDecoder(self):
        decoder = utils.ProtocolDecoder()
        self.assertFalse(decoder.completed())
        decoder.read(b"\0")
        self.assertFalse(decoder.completed())
        decoder.read(b"\0")
        self.assertFalse(decoder.completed())
        decoder.read(b"\0")
        self.assertFalse(decoder.completed())
        decoder.read(b"\0")
        self.assertTrue(decoder.completed())
        self.assertEqual(decoder.tag(), 0)
        self.assertEqual(decoder.len(), 0)

    def testDecoderShort(self):
        decoder = utils.ProtocolDecoder()
        self.assertFalse(decoder.completed())
        decoder.read(b"\0\1\0\2\0\1")
        self.assertTrue(decoder.completed())
        self.assertEqual(decoder.tag(), 1)
        self.assertEqual(decoder.len(), 2)
        self.assertEqual(struct.unpack('!h', decoder.data())[0], 1)

    def testDecoderLong(self):
        decoder = utils.ProtocolDecoder()
        self.assertFalse(decoder.completed())
        decoder.read(b"\0\2\0\4\0\1\2\3")
        self.assertTrue(decoder.completed())
        self.assertEqual(decoder.tag(), 2)
        self.assertEqual(decoder.len(), 4)
        self.assertEqual(struct.unpack('!l', decoder.data())[0], 0x010203)

    def test_driver(self):
        with self.connect() as drv:
            self.assertTrue(drv.check_alive())

    def test_driver_loop_10(self):
        for _ in range(0, 10):
            with self.connect() as drv:
                self.assertTrue(drv.check_alive())

    def test_reset_all_devices(self):
        with self.connect() as drv:
            self.assertTrue(drv.reset_all_devices())

    def test_driver_status(self):
        with self.connect() as drv:
            status = drv.get_device_status()
            self.assertTrue(status.cc)
            self.assertTrue(status.cl1)
            self.assertTrue(status.vmc_cl1_ready)
            self.assertTrue(status.vmc_cc_ready)

    def test_deposit_coin_and_perform_vend(self):
        with self.connect() as drv:
            status = drv.deposit_coin_and_perform_vend(2, 100, 3)
            print(status.msg)
            self.assertTrue(status.ok)

    def test_begin_session_and_perform_vend(self):
        with self.connect() as drv:
            status = drv.begin_session_and_perform_vend(1000, 500, 1)
            print(status.msg)
            self.assertTrue(status.ok)

    def test_cc_event(self):
        with self.connect() as drv:
            status = drv.cc_report_event(4)
            print(status.msg)
            self.assertTrue(status.ok)

    def test_power_reset(self):
        with self.connect() as drv:
            status = drv.power_reset()
            print(status.msg)
            self.assertTrue(status.ok)

    def test_send_5_cc_vends_sequentially(self):
        for _ in range(0, 5):
            with self.connect() as drv:
                status = drv.deposit_coin_and_perform_vend(2, 100, 3)
                self.assertTrue(status.ok)

    def test_send_5_cashless_vends(self):
        for _ in range(0, 5): 
            with self.connect() as drv:
                status = drv.begin_session_and_perform_vend(1000, 500, 1)
                print(status.msg)
                self.assertTrue(status.ok)




class DeviceStatuses(object):

    def __init__(self, flags):
        self.vmc_cc_ready = bool(flags[0])
        self.vmc_bl_ready = bool(flags[1])
        self.vmc_cl1_ready = bool(flags[2])
        self.vmc_cl2_ready = bool(flags[3])

        self.cc = bool(flags[4])
        self.bl = bool(flags[5])
        self.cl1 = bool(flags[6])
        self.cl2 = bool(flags[7])


class StatusMessage(object):

    def __init__(self, is_ok, msg):
        self.ok = is_ok
        self.msg = msg



import socket
import utils
import struct
import messages


class Driver(object):

    def __init__(self, ip, port=11000):
        self._is_open = False
        self._ip = ip
        self._port = port
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def __enter__(self):
        if not self._is_open:
            self._socket.connect((self._ip, self._port))
            self._is_open = True
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._is_open:
            self._socket.close()
            self._is_open = False


    def __read_status_message(self):
        decoder = utils.ProtocolDecoder()
        buf = bytes()
        while len(buf) != 4:
            missing = 4 - len(buf)
            tmpbuf = self._socket.recv(missing, socket.MSG_WAITALL)
            buf = buf + tmpbuf

        while not decoder.completed():
            decoder.read(buf)
            buf = self._socket.recv(decoder.len())

        if(decoder.tag() == 1):
            return messages.StatusMessage(True, "")
        elif decoder.tag() == 3:
            return messages.StatusMessage(False, str(decoder.data()))
        else:
            return messages.StatusMessage(False, "")


    def get_device_status(self):
        builder = utils.ProtocolBuilder(6)
        self._socket.send(builder.build())
        buf = self._socket.recv(12)
        decoder = utils.ProtocolDecoder()
        while not decoder.completed():
            decoder.read(buf)
        return messages.DeviceStatuses(struct.unpack('cccccccc', decoder.data()))

    def check_alive(self):
        builder = utils.ProtocolBuilder(1)
        self._socket.send(builder.build())
        return self.__read_status_message()

    def reset_all_devices(self):
        builder = utils.ProtocolBuilder(8)
        self._socket.send(builder.build())
        return self.__read_status_message()

    def deposit_coin_and_perform_vend(self, selection, selection_value, coin_type):
        builder = utils.ProtocolBuilder(9)
        builder.add_byte(selection)
        builder.add_short(selection_value)
        builder.add_byte(coin_type)
        self._socket.send(builder.build())
        return self.__read_status_message()

    def begin_session_and_perform_vend(self, credit, product_price, selection):
        builder = utils.ProtocolBuilder(11)
        builder.add_ushort(credit)
        builder.add_ushort(product_price)
        builder.add_ushort(selection)
        self._socket.send(builder.build())
        return self.__read_status_message()

    def cc_report_event(self, event):
        builder = utils.ProtocolBuilder(10)
        builder.add_byte(event)
        self._socket.send(builder.build())
        return self.__read_status_message()

    def power_reset(self):
        builder = utils.ProtocolBuilder(12)
        self._socket.send(builder.build())
        return self.__read_status_message()





import socket
import struct


class ProtocolDecoder(object):

    def __init__(self):
        self._buffer = []
        self._completed = False

    def completed(self):
            return self.header_complete() and (len(self._buffer) - 4) == self.len()

    def tag(self):
        return struct.unpack('!h', bytes(self._buffer[:2]))[0]

    def len(self):
        try:
            return struct.unpack('!h', bytes(self._buffer[2:4]))[0]
        except struct.error as e:
            raise e

    def header_complete(self):
        return len(self._buffer) >= 4

    def read(self, data):
        return self._buffer.extend(data)

    def data(self):
        return bytes(self._buffer[4:])


class ProtocolBuilder(object):

    def __init__(self, tag):
        self._tag = tag
        self._buffer = []
        self.add_short(tag)
        self.add_short(0)

    def _update_len(self):
        new_len = struct.pack('!H', len(self._buffer)-4);
        self._buffer[2] = new_len[0]
        self._buffer[3] = new_len[1]

    def add_bool(self, value):
        self._buffer.extend(struct.pack('c', b"\0" if not value else b"\1"))

    def add_byte(self, value):
        self._buffer.extend(struct.pack('c', bytes([value])))

    def add_char(self, c):
        self._buffer.append(bytes([c]))

    def add_short(self, s):
        self._buffer.extend(struct.pack('!h', int(s)))

    def add_long(self, s):
        self._buffer.extend(struct.pack('!l', int(s)))

    def add_ushort(self, s):
        self._buffer.extend(struct.pack('!H', int(s)))

    def add_ulong(self, s):
        self._buffer.extend(struct.pack('!L', int(s)))

    def build(self):
        self._update_len()
        return  bytes(self._buffer)

